	-- Echo_IIR_2b.vhd
-- Ian Clough 25 September 2007
-- UPDATED 4 October 2007
-- UPDATED 9 July 2010
-- UPDATED 9 September 2011
-- University of Queensland
-- Y(n) = a0*Xn + a1*Xn_1 + a2*Xn_2 - b1*Yn_1 - b2*Yn_2
-- Y(n) = a0*(Xn + Xn_2) + a1*Xn_1 - b1*Yn_1 - b2*Yn_2  ## optimised version

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;
entity Echo_IIR_2_B1 is port
   (
       DATAIN      : in  std_logic_vector(11 downto 0);
       DATAOUT      : out  std_logic_vector(11 downto 0);
       SAMPLECLK    : in   std_logic
   );
end Echo_IIR_2_B1;

architecture behavioral of Echo_IIR_2_B1 is
signal X : integer range -2048 to 2047 :=0;
signal Y : integer range -2048 to 2047 :=0;
-- *********** Powers of 2 for Denom *****************
-- 1048576  2097152 4194304  8388608 16777216 33554432 67108864 134217728
constant Denom: integer range 0 to 16777216:= 131072; --16777216;
constant k: integer range 1 to 256:=4; -- helps to prevent overflow
constant co_max: integer :=1048576;
constant co_min: integer :=0;
constant co_neg: integer := -co_max;
signal a0: integer range co_min to co_max-1;
signal a1: integer range co_min to co_max-1;
signal a2: integer range co_min to co_max-1;
signal b0: integer range co_neg to co_max-1;
signal b1: integer range co_neg to co_max-1;
signal b2: integer range co_neg to co_max-1;
type registers is array (0 to 2) of integer range -2048 to 2047;
signal Yreg : registers;
signal Xreg : registers;
begin

 ECHO:process (SAMPLECLK)

    begin
	   a0 <= 524288/k;
	  a1 <= 0/k;
	  a2 <= 0/k;
     b0 <= 0/k;
	  b1 <= 0/k;
     b2 <= 0/k;

     if falling_edge(SAMPLECLK) then
 Yreg(0) <= (a0*Xreg(0)/Denom + a2*Xreg(2)/Denom + a1*Xreg(1)/Denom) - (b1*Yreg(1)/Denom + b2*Yreg(2)/Denom);
	 Yreg(2) <= Yreg(1);
	 Yreg(1) <= Yreg(0);
	 Xreg(2) <= Xreg(1);
    Xreg(1) <= Xreg(0);
		end if;        --end fall
Xreg(0) <= to_INTEGER(unsigned(DATAIN))-2048 ;--change unipolar value (0 to 4095) to bipolar (-2048 to 2047)

    end process;
	 DATAOUT <=  STD_LOGIC_VECTOR(to_unsigned(Yreg(0)+2048,12));--convert unipolar integer to logic bit format
  end behavioral;


